#! /usr/bin/env python
# -*- coding: utf-8 -*-
####################
# Shinybow Matrix Plugin for Indigo
# Developed by Jeremy Swancoat

import os
import sys
import time
import serial

# Note the "indigo" module is automatically imported and made available inside
# our global name space by the host process.

##################################################################################################
class Plugin(indigo.PluginBase):

	##############################################################################################
	# Required Methods
	##############################################################################################
	##############################################################################################
	def __init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs):
		# Call the base class's init method
		indigo.PluginBase.__init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs)
		# Setting debug to True will show you verbose debugging information in the Indigo Event Log
		# LOTS of debugLog commands are placed throughout this routine.
		self.debug = None
		self.conn = None
		self.portEnabled = False
		self.commQueue = []
		self.zonedev = {}
			
	########################################
	def __del__(self):
		indigo.PluginBase.__del__(self)
		
	##############################################################################################
	# Non-Required Methods (but still defined by Indigo)
	##############################################################################################

	########################################
	def startup(self):
		# startup process for the serial device. Other than open the serial port, there should be none really. It just runs.
		self.debugLog(u"Startup Called")
		
		if self.conn is None:
			pass
		else:
			self.conn.close()		
			
		self.portEnabled = False
		serialUrl = self.getSerialPortUrl(self.pluginPrefs, u"serialport")
		self.debugLog("Serial Port URL is " + serialUrl)
		self.conn = self.openSerial(u"Shinybow Matrix", serialUrl, 9600, stopbits=1, timeout=0.1, writeTimeout=1)
		if self.conn is not None:
			self.portEnabled = True
			indigo.server.log("Serial Port Open at " + serialUrl)
	
	def shutdown(self):
		# close serial port here
		self.debugLog(u"Shutdown Called")
		self.conn.close()
		indigo.server.log("Serial Port Closed")


	
	def validateDeviceConfigUi(self, valuesDict, typeId, devId):
		# Validate that appropriate selections were made in device creation / modification dialogs.
		# Ensure that Zone is not already used
		self.debugLog("Validating Device")
		self.debugLog("Zone Selected: " + valuesDict["zoneselect"])
		###Logic to see if device is in use
		if valuesDict["zoneselect"] in self.zonedev:
			self.debugLog("Zone Already Assigned to Device")
			return(False, valuesDict)
		#Need to add logic to test if the device is already used.
		self.debugLog("Zone Available")	
		return (True, valuesDict)
		
	# This section creates entries and deletes entries in a dictionary of
	# OUTPUTxx , dev in an effort help find devices simply by what zone they control.		
	def deviceStartComm(self, dev):
		self.debugLog("deviceStartComm called")
		self.zonedev[dev.pluginProps["zoneselect"]] = dev
		self.debugLog("Just added: " + dev.pluginProps["zoneselect"] + " to zonedev")
		self.debugLog("Calling Status: " + dev.pluginProps["zoneselect"] + " ?")
		self.commQueue.append(dev.pluginProps["zoneselect"] + " ?")
		
	def deviceStopComm(self, dev):
		del self.zonedev[dev.pluginProps["zoneselect"]]
		self.debugLog("Just deleted: " + dev.pluginProps["zoneselect"] + " from zonedev")
		
	def validatePrefsConfigUi(self, valuesDict):
		
		errorsDict = indigo.Dict()		
		self.debugLog("Validating Serial Port")
		self.validateSerialPortUi(valuesDict, errorsDict, u"serialport")
		
		if valuesDict[u'showDebugInfo'] == True:
			self.debug = True
		else:
			self.debug = False
				
		
		if len(errorsDict) > 0:
			return (False, valuesDict, errorsDict)
		return (True, valuesDict)

		
	def closedPrefsConfigUi(self, valuesDict, userCancelled):
		# Basically, when the Config box closes, we want another chance to open the serial port.
		# Since that's all the startup method does, just call it from here.
		self.debugLog("closedPrefsConfigUI() called")
		
		#Obviously, if the box closes by cancelling, then do nothing.
		if userCancelled:
			pass
		else:
			self.startup()
		
	def runConcurrentThread(self):
		## This is what runs as the plugin is simply 'running'. 
		## Basically, if a command comes in from the serial device, you read it, parse it and hand it off to
		## whatever device/or variable needs to have its state updated.
		## In the meantime, you have a queue set up which gets loaded with commands from devices which must be fed
		## to the serial interface to be enacted.
		try:
			counter = 0
			while True:
				if self.portEnabled:
					#self.debugLog("Loop Running")
					sleeptime = 0.1
					fromswitch = 0
					fromswitch = self.conn.readline()
					#The Matrix does not report state on it's own. So query the matrix periodically to fetch all of the states.
					#Here send a request for all output states every 10 loops (1 sec)
					counter = counter + 1
					if counter >= 10:
						self.commQueue.append("OUTPUTALL ?")
						counter = 0
					if len(fromswitch) > 3:
						sleeptime = 0.01
						self.debugLog("Shinybow: " + fromswitch[:-1])
						self.parseToServer(fromswitch[:-1])  #removes last 1 character (;)
					if len(self.commQueue) > 0:
						sleeptime = 0.01
						# Sleep is set slow to keep processor use down, but putting 18 commands through with
						# 0.1 sleeptime takes almost 2 seconds, so if there's commands in the queue, drop the sleeptime, and
						# bang them through really fast. Then reset at beginning of loop.
						self.debugLog(self.commQueue[0])
						sendcount = self.conn.write(self.commQueue.pop(0) + ";")							
				self.sleep(sleeptime)
		except self.StopThread:
			pass	
		
	def stopConcurrentThread(self):
		self.stopThread = True
		
	def menuGetSources(self, filter, valuesDict, typeID, devID):
		sourcelist = []
		sourcelist.append(("Input00",self.pluginPrefs[u"Input00"]))
		sourcelist.append(("Input01",self.pluginPrefs[u"Input01"]))
		sourcelist.append(("Input02",self.pluginPrefs[u"Input02"]))
		sourcelist.append(("Input03",self.pluginPrefs[u"Input03"]))
		sourcelist.append(("Input04",self.pluginPrefs[u"Input04"]))
		sourcelist.append(("Input05",self.pluginPrefs[u"Input05"]))
		sourcelist.append(("Input06",self.pluginPrefs[u"Input06"]))
		sourcelist.append(("Input07",self.pluginPrefs[u"Input07"]))
		sourcelist.append(("Input08",self.pluginPrefs[u"Input08"]))
		return sourcelist
		
	def menuGetZones(self, filter, valuesDict, typeID, devID):
		zonelist = []
		for zonecode, servdev in self.zonedev.iteritems():
			zonelist.append((zonecode,servdev.name))
		return zonelist
		
	def parseToServer(self, fromswitch):
		#Looks Messy, but it's really just a bunch of tests to see what kind of statement it is
		#and then handling it - start with simple statements, so we can just move on from there.
		#All responses start with a '#' character, so start by stripping the first character.
		if fromswitch[:9] == "OutputAll":
			#Most responses from the switch should just be the status of all outputs (we call it a lot)
			#The response looks like OutputAll 0102030002030605 where the 16 digit number is two-digit input
			#numbers for each of the 8 zones.
			#put a method here that basically loops through the zones and plucks out the current input
			#from the list and sets the input correctly.
			fromswitch = fromswitch[-16:]
			for x in range (1,8):
				zonecode = "OUTPUT0"+str(x)
				#take the first two characters which represent the input for the relevant zone...
				inputcode=fromswitch[:2]
				#... and then remove those characters from the list, queing up the variable for the next zone
				fromswitch=fromswitch[2:]
				if zonecode in self.zonedev:
					#We want to report input changes in the log, but OutputAll reports will be frequent,
					#so we will only report changes. Before changing anything on the server, do a check on it's
					#current state, and only change and report if it's different.
					self.debugLog(zonecode + " found in devices list")
					servdev = self.zonedev[zonecode]
					savedinput = {}
					fullinput = "Input"+inputcode
					dev = self.zonedev[zonecode]
					savedinput[zonecode] = dev.states["Input"]
					if savedinput[zonecode] != fullinput:
						servdev.updateStateOnServer("Input", value=fullinput)
						indigo.server.log(servdev.name + " set to Input " + inputcode)
				else:
					self.debugLog(zonecode + " not yet defined")
		elif fromswitch[:6] == "Output":
			#Sometimes we'll see just a single Output response, i.e. Output04 01 means Output 4 is on input 1.
			#Simply update the device state.
			zonecode=fromswitch[:8]
			if zonecode in self.zonedev:
				self.debutLog(zonecode + " found in devices list")
				servdev = self.zonedev[zonecode]
				fullinput = "Input"+fromswitch[-2:]
				servdev.updateStateOnServer("Input", value=fullinput)
				self.debugLog(servdev.name+" set to Input "+fromswitch[:-2])
			else:
				self.debugLog(zonecode + " not yet defined")
		elif fromswitch == "SB5688 01":
			#This response is what the switch kicks out everytime it gets an unknown command. Ack and log it so that
			#at least we know the plugin is sending some invalid stuff.
			indigo.server.log("Invalid Command sent to Matrix")
		else:
			self.debugLog("No method defined to process response from Matrix")
			
	def setZoneSource(self, pluginAction, dev):

		indigo.server.log("Setting Source of " + dev.name + " to " + pluginAction.props.get(u"reqzone"))	
		self.commQueue.append(dev.pluginProps["zoneselect"] + " " + pluginAction.props.get(u"reqzone")[-2:])
		